<?php
/*
Plugin Name: Flx Woo
Plugin URI:  https://wordpress.org/plugins/flx-woo/
Description: Convert WooCommerce into Headless with SaaS.
Version:     1.2.0
Author:      Rickey Gu
Author URI:  https://flxwoo.com
License:     AGPL-3.0+
License URI: https://www.gnu.org/licenses/agpl-3.0.en.html
Text Domain: flx-woo
Domain Path: /languages
*/

if ( ! defined( 'ABSPATH' ) )
{
	exit;
}

define( 'FLX_WOO_PATH', plugin_dir_path( __FILE__ ) );

include_once ABSPATH . 'wp-admin/includes/plugin.php';

require_once FLX_WOO_PATH . 'cfg/cfg.php';
require_once FLX_WOO_PATH . 'flx/flx.php';

class FlxWoo
{
	private $home_url = '';
	private $home_url_pattern = '';

	private $page_url = '';

	private $flx = NULL;

	private $shop_page_url = '';
	private $shop_page_url_pattern = '';


	public function __construct()
	{
		$this->home_url = home_url();
		$this->home_url_pattern = str_replace( [ '/', '.' ], [ '\/', '\.' ], $this->home_url );

		$parts = wp_parse_url( $this->home_url );
		$page_url = $parts['scheme'] . '://' . $parts['host'] . add_query_arg( [] );
		$page_url = preg_replace( '/#.*$/', '', $page_url );
		$this->page_url = preg_replace( '/^' . $this->home_url_pattern . '/', '', $page_url );

		$this->flx = new FlxWooFlx();
	}

	public function __destruct()
	{
	}


	public function init()
	{
		$shop_page_id = wc_get_page_id( 'shop' );
		$shop_page_url = get_permalink( $shop_page_id );

		$shop_page_url = preg_replace( '/^' . $this->home_url_pattern . '/', '', $shop_page_url );
		$this->shop_page_url = preg_replace( '/\/$/', '', $shop_page_url );
		$this->shop_page_url_pattern = str_replace( [ '/', '.' ], [ '\/', '\.' ], $this->shop_page_url );
	}


	private function get_route( $page_url )
	{
		global $wp_query;

		$route = '';

		if ( $wp_query->is_404 )
		{
			$route = '/404/';
		}
		elseif ( $wp_query->is_home )
		{
			if ( is_front_page() )
			{
				$route = '/';
			}
		}
		elseif ( is_shop() )
		{
			if ( ! empty( $this->shop_page_url_pattern ) )
			{
				if ( preg_match( '/^' . $this->shop_page_url_pattern . '\//i', $page_url ) )
				{
					$route = '/shop/';
				}
			}
		}
		elseif ( is_product() )
		{
			if ( preg_match( '/\/([\w|\d|-]+)\/$/i', $this->page_url, $matches ) )
			{
				$slug = $matches[1];
				$route = '/product/' . $slug;
			}
		}
		elseif ( is_cart() )
		{
			$route = '/cart/';
		}
		elseif ( is_checkout() )
		{
			$route = '/checkout/';
		}

		return $route;
	}

	private function get_body( $route )
	{
		$body = [
			'version' => FLX_WOO_VERSION,
			'homeUrl' => $this->home_url,
			'pageUrl' => $this->page_url
		];

		if ( '/shop/' === $route )
		{
			$settings = [];

			$posts_per_page = get_option( 'posts_per_page' );
			if ( FALSE !== $posts_per_page )
			{
				$settings['postsPerPage'] = $posts_per_page;
			}

			$body['settings'] = $settings;


			$pageInfo = [];

			if ( ! empty( $this->shop_page_url_pattern ) )
			{
				if ( preg_match( '/^' . $this->shop_page_url_pattern . '\/\?startCursor=([0-9a-zA-Z]{24})$/i', $this->page_url, $matches ) )
				{
					$pageInfo['startCursor'] = $matches[1];
				}
				elseif ( preg_match( '/^' . $this->shop_page_url_pattern . '\/\?endCursor=([0-9a-zA-Z]{24})$/i', $this->page_url, $matches ) )
				{
					$pageInfo['endCursor'] = $matches[1];
				}
			}

			$body['pageInfo'] = $pageInfo;
		}

		return $body;
	}

	private function page_callback( $page )
	{
		if ( preg_match( '/\.js$/i', $this->page_url ) )
		{
			return $page;
		}

		if ( ! preg_match( '/^<!DOCTYPE html>/i', trim( $page ) ) )
		{
			return $page;
		}

		$route = $this->get_route( $this->page_url );
		if ( empty( $route ) )
		{
			return $page;
		}

		$body = $this->get_body( $route );
		if ( empty( $body ) )
		{
			return $page;
		}

		$new_page = $this->flx->get_page( $route, $body );
		if ( empty( $new_page ) )
		{
			return $page;
		}

		return $new_page;
	}


	public function after_setup_theme()
	{
		ob_start( [ $this, 'page_callback' ] );
	}

	public function shutdown()
	{
		if ( FALSE !== ob_get_length() )
		{
			ob_end_flush();
		}
	}


	public function settings_link( $links )
	{
		$url = get_admin_url();
		$url .= 'admin.php';
		$url = add_query_arg( 'page', 'flx-woo', $url );

		$link = '<a href="' . esc_url( $url ) . '">' . __( 'Settings', 'flx-woo' ) . '</a>';
		array_push( $links, $link );

		return $links;
	}
}


if ( ! is_plugin_active( 'woocommerce/woocommerce.php' ) || ! is_plugin_active( 'wp-graphql/wp-graphql.php' ) || ! is_plugin_active( 'wp-graphql-woocommerce/wp-graphql-woocommerce.php' ) || ! ( is_plugin_active( 'wp-graphql-cors/wp-graphql-cors.php' ) || is_plugin_active( 'wp-graphql-cors-master/wp-graphql-cors.php' ) ) )
{
	return;
}
elseif ( 'admin-ajax.php' === $GLOBALS['pagenow'] || 'wp-activate.php' === $GLOBALS['pagenow'] || 'wp-cron.php' === $GLOBALS['pagenow'] || 'wp-login.php' === $GLOBALS['pagenow'] || 'wp-signup.php' === $GLOBALS['pagenow'] )
{
	return;
}

include_once FLX_WOO_PATH . 'admin/admin.php';

$flx_woo = new FlxWoo();

if ( ! is_admin() )
{
	add_action( 'init', [ $flx_woo, 'init' ] );

	add_action( 'after_setup_theme', [ $flx_woo, 'after_setup_theme' ] );
	add_action( 'shutdown', [ $flx_woo, 'shutdown' ] );
}

add_filter( 'plugin_action_links_flx-woo/flx-woo.php', [ $flx_woo, 'settings_link' ] );
